import asyncio
import websockets
import aiohttp
import json
import logging

# 配置日志记录器
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(levelname)s - %(message)s',
    handlers=[
        logging.FileHandler('websocket_server.log'),
        logging.StreamHandler()
    ]
)

async def get_streaming_response(content):
    url = "https://api.moonshot.cn/v1/chat/completions"
    headers = {
        "Authorization": "Bearer sk-GV2nmITmtitcfnmyBdUbahO95XTFznByJjjfUbMG6bMJEWrT",
        "Accept": "text/event-stream",
        "Content-Type": "application/json",
    }
    payload = {
        "model": "moonshot-v1-8k",
        "messages": [
            {"role": "system", "content": "你是旅行小助手，一个擅长所有旅游相关信息的人工智能助手，你更擅长中文和英文的对话。你会为用户提供安全，有帮助，准确的回答。同时，你会拒绝一切涉及恐怖主义，种族歧视，黄色暴力等问题的回答接下来我会问你一些旅游相关的问题，如果问题较简单，回答可以控制在一百字以内，如果问题较复杂，回答可以控制在三百字以内，如果问题和旅游无任何关联可以提示用户对旅游相关问题进行提问"},
            {"role": "user", "content": content}
        ],
        "temperature": 0.3,
        "stream": True 
    }
    async with aiohttp.ClientSession() as session:
        async with session.post(url, json=payload, headers=headers) as response:
            if response.content_type == 'text/event-stream':
                async for line in response.content:
                    line = line.decode().strip()  # Decode bytes to string and strip whitespace
                    if line and line.startswith('data:'):
                        data_line = line[len('data: '):]  # Remove 'data: ' prefix
                        logging.info(f"Received raw data: {data_line}")  # Log the raw data
                        try:
                            chunk = json.loads(data_line)
                            content = chunk.get("choices", [{}])[0].get("delta", {}).get("content", "")
                            if content:  # Only yield content if it's not empty
                                yield content
                        except json.JSONDecodeError as e:
                            logging.error(f"JSON decode error: {e}")
                            logging.error(f"Invalid JSON data: {data_line}")
                        except Exception as e:
                            logging.error(f"Unexpected error: {e}")
                            logging.error(f"Error data: {data_line}")
                    else:
                        logging.info("Received empty or non-data line, skipping...")
            else:
                logging.error(f"Unexpected content type: {response.content_type}")
                raise ValueError("Expected an event stream response")

async def handler(websocket, path):
    try:
        # 接收 WebSocket 消息的异步循环
        async for message in websocket:
            # 打印从微信小程序接收到的 content 内容
            logging.info(f"Received content from WeChat mini program: {message}")
            
            # 调用 get_streaming_response 函数，并将接收到的消息作为参数
            async for response_content in get_streaming_response(message):
                # 发送响应数据给 WebSocket 客户端
                await websocket.send(response_content)
                logging.info(f"Sent to client: {response_content}")
    except Exception as e:
        logging.error(f"Error in streaming: {e}")
    finally:
        await websocket.close()

start_server = websockets.serve(handler, "localhost", 5678)

asyncio.get_event_loop().run_until_complete(start_server)
asyncio.get_event_loop().run_forever()
